/*
 *	Dashboard.mqh
 *
 *	Basic dashboard class, limited options
 *
 *	Copyright 2020, Orchard Forex
 *	https://www.orchardforex.com
 *
 */

#property copyright "Copyright 2020, Orchard Forex"
#property link      "https://www.orchardforex.com"
#property version   "1.00"
#property strict

struct SDashboardRow {

	string	Text;
	color		TextColor;
	string	Font;
	int		Size;
	
	int		YSize;
	int		YDistance;

};

class CDashboard {

private:	// Member variables

	SDashboardRow	mRows[];					//	An array to hold the display text
	
	string			mName;					//	Keep this unique to use more than one dashboard at a time
	int				mYDistance;				//	Use this to avoid clashing with other text at top of screen
	
private:	//	Functions
	
	string			TextObjectName(int index)	{	return(StringFormat("%s_%i", mName, index));	}	//	The object name of a single line
	int				CalcRowYDistance(int index);																	//	Calculate y distance for a single row
	void				SetRowYDistance(int index);																	//	Set the y distance for a single row
	void				DrawRow(int row);																					//	Display a single row on screen
	void				DeleteAll();																						//	Delete all rows and objects

public:	// Constructors and initialisers

	//CDashboard()																											//	Not using a default constructor
	CDashboard(string name, int yDistance)		{	Init(name, yDistance);								}
	~CDashboard()										{	DeleteAll();											}

	void				Init(string name, int yDistance);

public:	//	CRUD - Create, Read, Update, Delete

	//	Create - To just add new rows at the end
	void				AddRow(string text, color textColor, string font, int size);
	
	//	Read - To get row information
	string			GetRowText(int index)		{	return(mRows[index].Text);				}
	color				GetRowTextColor(int index)	{	return(mRows[index].TextColor);		}
	string			GetRowFont(int index)		{	return(mRows[index].Font);				}
	int				GetRowSize(int index)		{	return(mRows[index].Size);				}
	int				GetRowYDistance(int index)	{	return(mRows[index].YDistance);		}

	//	Update - To Update row text or other information
	void				SetRow(int index, string text, color textColor, string font, int size);
	void				SetRowText(int index, string text);

	//	Delete - To remove rows
	void				DeleteRows(int first, int last);		// Delete rows and the associated objects

public:		//	Properties

	string			GetName()						{	return(mName);								}
	
	int				GetYDistance()					{	return(mYDistance);						}
	void				SetYDistance(int value);
	   
};

//+------------------------------------------------------------------+
//	Constructors and Initialisers
//+------------------------------------------------------------------+

void		CDashboard::Init(string name, int yDistance) {

	//
	//	Standard initialisation routine
	//

	mName						=	name;				//	First set the name of the dashboard

	DeleteAll();									//	In case there are any old objects remaining on screen

	SetYDistance(yDistance);					//	Needs special attention to update existing rows
														//		Not really needed on init because there are no rows
														//		but I maintain a consistent approach
	
}

//+------------------------------------------------------------------+
//	General functions
//+------------------------------------------------------------------+

int		CDashboard::CalcRowYDistance(int index) {

	return( (index==0) ? mYDistance : mRows[index-1].YDistance + mRows[index-1].YSize + 2);
		// 2 = gap between lines

}

void		CDashboard::SetRowYDistance(int index) {

	//
	//	Set the y distance for all rows beginning with first
	//
	int	yDistance;
	for (int i=index; i<ArraySize(mRows); i++) {
		yDistance					=	CalcRowYDistance(i);
		if (yDistance!=mRows[i].YDistance) {			//	Only make the change if needed, save resources
			mRows[i].YDistance	=	yDistance;
			DrawRow(i);
		} else {
			break;
		}
	}
	
	return;
	
}

void		CDashboard::DrawRow(int index) {

	string	objectName	=	TextObjectName(index);
	if (ObjectFind(objectName)<0) {
		ObjectCreate(objectName, OBJ_LABEL, 0, 0, 0);
	}

	int	currentYSize			=	mRows[index].YSize;		//	In case of change

	ObjectSetText(objectName, mRows[index].Text, mRows[index].Size, mRows[index].Font, mRows[index].TextColor );
	ObjectSet(objectName, OBJPROP_CORNER, 1);   			// controls the corner the text is put into 0=top left 1=topright 2=bottom left 3=bottom right
	ObjectSet(objectName, OBJPROP_XDISTANCE, 8);			//controls distance text block is from margin  
	ObjectSet(objectName, OBJPROP_YDISTANCE, mRows[index].YDistance);

	int	newYSize					=	(int)ObjectGetInteger(0, objectName, OBJPROP_YSIZE);
	if (newYSize>0) {
		mRows[index].YSize	=	newYSize;
	} else if (mRows[index].YSize==0) {
		mRows[index].YSize	=	mRows[index].Size*96/72;	//	Rough points to pixel scaling
	}

	if (currentYSize!=mRows[index].YSize) {					//	Size changed, Need to ripple the position update
		SetRowYDistance(index+1);									//	Also redraws each row
																			//	I know this is sloppy and can cause multiple redraws but it's OK for minimal use
	}

}

void		CDashboard::DeleteAll() {

	DeleteRows(0, ArraySize(mRows)-1);		//	Takes care of deleting everything in the array
	ObjectsDeleteAll(0, mName+"_");			//	In case any on screen objects weren't caught

}

//+------------------------------------------------------------------+
//	Create functions
//+------------------------------------------------------------------+

void	CDashboard::AddRow(string text, color textColor, string font, int size) {

	//
	//	Adds a new row to the end of the array and displays
	//
		
	int	index					=	ArraySize(mRows);		//	This will be the index of the new row
	ArrayResize(mRows, index+1);							//	Increase the size of the array for the new row
	
	mRows[index].Text			=	text;						//	Just set the properties of the row
	mRows[index].TextColor	=	textColor;
	mRows[index].Font			=	font;					
	mRows[index].Size			=	size;
	SetRowYDistance(index);									//	This value is calculated based on index
	
	DrawRow(index);											//	Draw the text on screen

	return;
	
}    

//+------------------------------------------------------------------+
//	Update functions
//+------------------------------------------------------------------+

void		CDashboard::SetRow(int index, string text, color textColor, string font, int size) {

	//
	//	Set all values for the row
	//	Not using individual properties here to avoid multiple redraws
	//
	
	mRows[index].Text					=	text;
	mRows[index].TextColor			=	textColor;
	mRows[index].Font					=	font;
	mRows[index].Size					=	size;

	DrawRow(index);													//	Redraw the row
	
	return;
	
}

void		CDashboard::SetRowText(int index, string text) {

	//
	//	Just update the text
	//
	
	mRows[index].Text	=	text;
	DrawRow(index);									//	Redraw the row

	return;
	
}

void		CDashboard::SetYDistance(int value) {

	//
	//	Sets the y distance for the dashboard
	//
	mYDistance	=	value;
	SetRowYDistance(0);		//	When this value changes all rows would move
	
}

//+------------------------------------------------------------------+
//	Delete functions
//+------------------------------------------------------------------+

void		CDashboard::DeleteRows(int first, int last) {

	//
	//	Delete a range of rows from first to last
	//
	
	if (last>=ArraySize(mRows))	last	=	ArraySize(mRows)-1;
	if (first<0)						first	=	0;
	if (last<first)	return;
	
	// This is a quick and easy way, it may be visible on screen but
	//		the function is unlikely to be used too often
	
	SDashboardRow	rows[];							//	Used as a temporary copy

	ArrayResize(rows, ArraySize(mRows));
	for (int i = 0; i<ArraySize(mRows); i++) {
		rows[i]	=	mRows[i];						//	Making a copy of mRows
	}

	ObjectsDeleteAll(0, mName+"_");				//	Remove everything from screen
	ArrayResize(mRows, 0);							//	Clean up the current array

	for (int i = 0; i<ArraySize(rows); i++) {	//	Copy everything back
		if (i<first || i>last) {					//	Skip the rows to be removed
			AddRow(rows[i].Text, rows[i].TextColor, rows[i].Font, rows[i].Size);
															//	Using the functions we already have
		}
	}
	
	return;
	
}


