/*
 *	Dashboard.mqh
 *
 *	Basic dashboard class, limited options
 *
 *	Copyright 2020, Orchard Forex
 *	https://www.orchardforex.com
 *
 */
 
/*
 *	Version History
 *
 *	V2
 *		Added error checks for invalid index numbers
 *		Ability to set the corner and horizontal margin
 *		Some additional property settings
 *
 *	V1
 *		Original version
 *
 */

#property copyright "Copyright 2020, Orchard Forex"
#property link      "https://www.orchardforex.com"
#property version   "1.00"
#property strict

struct SDashboardRow {

	string	Text;
	color		TextColor;
	string	Font;
	int		Size;
	
	int		YSize;
	int		YDistance;

};

class CDashboard {

private:	// Member variables

	SDashboardRow	mRows[];					//	An array to hold the display text
	
	//	V2 - added corner and xdistance
	string			mName;					//	Keep this unique to use more than one dashboard at a time
	int				mCorner;					//	Corner from 0-3
	int				mXDistance;				//	Margin from side of screen
	int				mYDistance;				//	Use this to avoid clashing with other text at top of screen

private:	//	Functions
	
	string			TextObjectName(int index)		{	return(StringFormat("%s_%i", mName, index));	}	//	The object name of a single line

	// V2 - New SetSingleRowYDistance
	int				CalcRowYDistance(int index);																		//	Calculate y distance for a single row
	void				SetRowYDistance(int index);																		//	Set the y distance for a single row
	bool				SetSingleRowYDistance(int index);																//	Set the y distance with no ripple update

	//	V2 - New DrawAll and DrawSingleRow
	void				DrawAll();																								//	Redraw all rows
	void				DrawRow(int row);																						//	Display a single row on screen
	bool				DrawSingleRow(int row);																				//	Display a single row but do not push any updates

	void				DeleteAll();																							//	Delete all rows and objects

	//	V2 - New to handle corner and index validation
	bool				IsTopCorner()						{	return(mCorner==0 || mCorner==1);				}	// Used often
	bool				IsValidIndex(int index)			{	return(index>=0 && index<ArraySize(mRows));	}	//	To validate function arguments

private:	//	V2 - General purpose properties - new to simplify coding

	template <typename T> bool	SetValue(T &var, T value);															//	Allows shorthand in the code
	template <typename T> bool	SetRowValue(int index, T &var, T value);										//	same for row specific
	
public:	// Constructors and initialisers

	//CDashboard()																												//	Not using a default constructor
	//	V2 - New constructor added to maintain compatibility with version 1
	//		Aso changed the way the code is written, removing inline call to Init, see below
	CDashboard(string name, int yDistance)			{	Init(name, 1, 8, yDistance); 						}
	CDashboard(string name, int corner, int xDistance, int yDistance);
	~CDashboard()											{	DeleteAll();											}

	//	V2 - Updated to add corner and xdistance
	//	Original kept for compatibility
	void				Init(string name, int yDistance)
																{	Init(name, mCorner, mXDistance, yDistance);	}
	void				Init(string name, int corner, int xDistance, int yDistance);

public:	//	CRUD - Create, Read, Update, Delete

	//	Create - To just add new rows at the end
	void				AddRow(string text, color textColor, string font, int size);
	
	//	Read - To get row information
	string			GetRowText(int index)		{	return(mRows[index].Text);				}
	color				GetRowTextColor(int index)	{	return(mRows[index].TextColor);		}
	string			GetRowFont(int index)		{	return(mRows[index].Font);				}
	int				GetRowSize(int index)		{	return(mRows[index].Size);				}
	int				GetRowYDistance(int index)	{	return(mRows[index].YDistance);		}

	//	Update - To Update row text or other information
	//	V2 - Added some more set functions, color, font, size - optional
	void				SetRow(int index, string text, color textColor, string font, int size);
	void				SetRowText(int index, string text);
	void				SetRowColor(int index, color clr);
	void				SetRowFont(int index, string font);
	void				SetRowSize(int index, int size);

	//	Delete - To remove rows
	void				DeleteRows(int first, int last);		// Delete rows and the associated objects

public:		//	Properties

	string			GetName()						{	return(mName);								}

	//	V2 - get corner and xdistance are new	
	int				GetCorner()						{	return(mCorner);							}
	void				SetCorner(int value);
	
	int				GetXDistance()					{	return(mXDistance);						}
	void				SetXDistance(int value);

	int				GetYDistance()					{	return(mYDistance);						}
	void				SetYDistance(int value);
	   
};

//+------------------------------------------------------------------+
//	Constructors and Initialisers
//+------------------------------------------------------------------+

//	V2 - this constructor will set default values
CDashboard::CDashboard(string name,int corner,int xDistance,int yDistance)
								:	mCorner(1)	{
	
	Init(name, corner, xDistance, yDistance);

}

//	V2 - revised init takes additional arguments
//	originl compatible call is handled in the declaration above
void		CDashboard::Init(string name, int corner, int xDistance, int yDistance) {

	//
	//	Standard initialisation routine
	//

	mName						=	name;				//	First set the name of the dashboard

	DeleteAll();									//	In case there are any old objects remaining on screen

	//	V2 - new corner and xdistance
	SetCorner(corner);							//	The set functions have validation built in
	SetXDistance(xDistance);
	SetYDistance(yDistance);
	
}

//+------------------------------------------------------------------+
//	General functions
//+------------------------------------------------------------------+

//	V2 - corner changes
int		CDashboard::CalcRowYDistance(int index) {

	//	Different result if the corner is top or bottom
	if (IsTopCorner()) {
		return( (index==0) ? mYDistance : mRows[index-1].YDistance + mRows[index-1].YSize + 2);
	} else {
		return( (index==(ArraySize(mRows)-1)) ? mYDistance : mRows[index+1].YDistance + mRows[index+1].YSize + 2);
	}

}

//	V2 - corner changes, leads to the set single row function
void		CDashboard::SetRowYDistance(int index) {

	//
	//	Set the y distance for all rows beginning with index
	//
	if (IsTopCorner()) {
		for (int i=index; i<ArraySize(mRows); i++) {
			if (!SetSingleRowYDistance(i)) break;
		}
	} else {
		for (int i=index; i>=0; i--) {
			if (!SetSingleRowYDistance(i)) break;
		}
	}
	
	return;
	
}

//	V2 - new
bool		CDashboard::SetSingleRowYDistance(int index) {

	//
	//	Just sets the Y Distance for one row, no ripple
	//
	
	if (!IsValidIndex(index))	return(false);
	
	int	yDistance			=	CalcRowYDistance(index);
	return(SetRowValue(index, mRows[index].YDistance, yDistance));

}

//	V2 - new needed for corner changes
void		CDashboard::DrawAll()	{

	//
	//	Force each row to be drawn again
	//	Recalculate the y position before drawing the row
	//	If this causes one of the cascading redraws then there is no harm
	//
	if (IsTopCorner()) {	//	Different direction for top or bottom corner
		for (int i = 0; i<ArraySize(mRows); i++) {
			SetSingleRowYDistance(i);
			DrawRow(i);
		}
	} else {
		for (int i = ArraySize(mRows)-1; i>=0; i--) {
			SetSingleRowYDistance(i);
			DrawRow(i);
		}
	}
	
}

//	V2 - simplified for general use
void		CDashboard::DrawRow(int index) {

	//
	//	Draw the row at index, if YSize changes force the next row to reposition
	//		The reposition would most likely cause all following rows to be redrawn
	//
	if (!IsValidIndex(index))	return;

	if (DrawSingleRow(index)) {									//	Size changed, Need to ripple the position update
		if (IsTopCorner()) {
			SetRowYDistance(index+1);								//	Also redraws each row
		} else {
			SetRowYDistance(index-1);
		}
	}

}

//	V2 - new separates single row from loops, easier for corner changes
bool		CDashboard::DrawSingleRow(int index) {

	//
	//	Create or update the screen object at index with the values of mRows
	//	Return true if the YSize changes because that indicates an impact
	//		to positioning of other rows
	//
	if (!IsValidIndex(index))	return(false);

	string	objectName	=	TextObjectName(index);
	if (ObjectFind(objectName)<0) {
		ObjectCreate(objectName, OBJ_LABEL, 0, 0, 0);
	}

	ObjectSetText(objectName, mRows[index].Text, mRows[index].Size, mRows[index].Font, mRows[index].TextColor );
	ObjectSet(objectName, OBJPROP_CORNER, mCorner);   						// controls the corner the text is put into 0=top left 1=topright 2=bottom left 3=bottom right
	ObjectSet(objectName, OBJPROP_XDISTANCE, mXDistance);					//controls distance text block is from side margin  
	ObjectSet(objectName, OBJPROP_YDISTANCE, mRows[index].YDistance);	//controls distance text block is from top or bottom margin  

	int	newYSize			=	(int)ObjectGetInteger(0, objectName, OBJPROP_YSIZE);
	if (newYSize==0) {																//	Can return zero if a problem, just go with it
		if (mRows[index].YSize==0) {												//	0 = hasn't been set yet
			newYSize			=	mRows[index].Size*96/72;						//	Rough default points to pixel scaling
		} else {
			return(false);																// Size doesn't change
		}
	}
	
	if (newYSize!=mRows[index].YSize) {											//	Can't use SetRowValue here
		mRows[index].YSize	=	newYSize;										//	It would cause a loop
		return(true);																	//	Not an infinite loop but might as well avoid it
	}
	return(false);

}

//	V2 - fix the mistake
void		CDashboard::DeleteAll() {

	//
	//	Reset the mRows array and remove all associated objects from the screen
	//
	//	I made a careles mistake here earlier.
	//	Everything still worked but very bad code
	ArrayResize(mRows, 0);						//	Wipe out the existing rows
	ObjectsDeleteAll(0, mName+"_");			//	In case any on screen objects weren't caught

}

//	V2 - templating techniques
//+------------------------------------------------------------------+
//	Templated set functions
//+------------------------------------------------------------------+
template <typename T> bool		CDashboard::SetValue(T &var, T value) {

	//
	//	Just a shorthand function to update a value if needed and returns
	//		a condition if set
	//
	if (var!=value) {
		var	=	value;
		DrawAll();
		return(true);
	}
	return(false);
	
}

template <typename T> bool		CDashboard::SetRowValue(int index, T &var, T value) {

	//
	//	Another shorthand to update a value if needed and then redraw
	//	No need to check index for valid because the indexed value is already passed in
	//
	if (var!=value) {
		var	=	value;
		DrawRow(index);
		return(true);
	}
	return(false);
	
}

//+------------------------------------------------------------------+
//	Create functions
//+------------------------------------------------------------------+

void	CDashboard::AddRow(string text, color textColor, string font, int size) {

	//
	//	Adds a new row to the end of the array and displays
	//	The end of the array is always at the bottom, regardless of corner
	//
		
	int	index					=	ArraySize(mRows);		//	V2 - This will be the index of the new row
	ArrayResize(mRows, index+1);							//	Increase the size of the array for the new row
	
	mRows[index].Text			=	text;						//	Just set the properties of the row
	mRows[index].TextColor	=	textColor;
	mRows[index].Font			=	font;					
	mRows[index].Size			=	size;
	SetRowYDistance(index);									//	This value is calculated based on index
	
	DrawRow(index);											//	Draw the text on screen

	return;
	
}    

//+------------------------------------------------------------------+
//	Update functions
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//	First the row only functions
//+------------------------------------------------------------------+
void		CDashboard::SetRow(int index, string text, color textColor, string font, int size) {

	//
	//	Set all values for the row
	//	Not using individual properties here to avoid multiple redraws
	//
	
	if (!IsValidIndex(index))	return;	//	V2

	mRows[index].Text					=	text;
	mRows[index].TextColor			=	textColor;
	mRows[index].Font					=	font;
	mRows[index].Size					=	size;

	DrawRow(index);											//	Redraw the row
	
	return;
	
}

void		CDashboard::SetRowText(int index, string text) {

	//
	//	Just update the text
	//
	
	if (!IsValidIndex(index))	return;					//	V2 - can't be added to the template
	SetRowValue(index, mRows[index].Text, text);		//	and using the template

	return;
	
}

//	V2 - color font and size more of the same
void		CDashboard::SetRowColor(int index,color clr) {

	//
	//	Update color
	//
	
	if (!IsValidIndex(index))	return;
	SetRowValue(index, mRows[index].TextColor, clr);

	return;
	
}

void		CDashboard::SetRowFont(int index,string font) {

	//
	//	Update the font
	//
	
	if (!IsValidIndex(index))	return;
	SetRowValue(index, mRows[index].Font, font);

	return;
	
}

void		CDashboard::SetRowSize(int index,int size) {

	//
	//	Update the size
	//
	
	if (!IsValidIndex(index))	return;
	SetRowValue(index, mRows[index].Size, size);

	return;
	
}

//+------------------------------------------------------------------+
//	The dashboard global updates
//+------------------------------------------------------------------+
void		CDashboard::SetCorner(int value) {

	//
	//	Move the entire block to another corner
	//	Forces redraw of everything
	//
	if (value<0 || value>3) return;						//	Invalid corner (see changes to constructor)
	
	SetValue(mCorner, value);								//	Recalculate YDistance and redraw
	
}

void		CDashboard::SetXDistance(int value) {

	//
	//	Sets the x distance for the dashboard
	//
	SetValue(mXDistance, value);
	
}

void		CDashboard::SetYDistance(int value) {

	//
	//	Sets the y distance for the dashboard
	//
	SetValue(mYDistance, value);
	
}

//+------------------------------------------------------------------+
//	Delete functions
//+------------------------------------------------------------------+

void		CDashboard::DeleteRows(int first, int last) {

	//
	//	Delete a range of rows from first to last
	//
	
	if (last>=ArraySize(mRows))	last	=	ArraySize(mRows)-1;
	if (first<0)						first	=	0;
	if (last<first)	return;
	
	// This is a quick and easy way, it may be visible on screen but
	//		the function is unlikely to be used too often
	
	SDashboardRow	rows[];							//	Used as a temporary copy

	ArrayResize(rows, ArraySize(mRows));
	for (int i = 0; i<ArraySize(mRows); i++) {
		rows[i]	=	mRows[i];						//	Making a copy of mRows
	}

	ObjectsDeleteAll(0, mName+"_");				//	Remove everything from screen
	ArrayResize(mRows, 0);							//	Clean up the current array

	for (int i = 0; i<ArraySize(rows); i++) {	//	Copy everything back
		if (i<first || i>last) {					//	Skip the rows to be removed
			AddRow(rows[i].Text, rows[i].TextColor, rows[i].Font, rows[i].Size);
															//	Using the functions we already have
		}
	}
	
	return;
	
}


