/*
 *	DashboardV3.mqh
 *
 *	Dashboard in a separate sub window panel
 *
 *	Copyright 2020, Orchard Forex
 *	https://www.orchardforex.com
 *
 */

/*
 *	Version History
 *
 *	Version 2
 *		2.1	Add checking for invalid index numbers
 *		2.2	Add corner selection and xdistance
 *
 *	Version 3
 *		3.1	Draw in a second panel
 *
 */

#property copyright "Copyright 2020, Orchard Forex"
#property link      "https://www.orchardforex.com"
#property version   "1.00"
#property strict

#include "DashboardColumn.mqh"

class CDashboardV3 {

private:	// Member variables

	CDashboardColumn	*mColumns[];			//	An array to hold the columns
	
	string			mName;					//	Keep this unique to use more than one dashboard at a time
	int				mSubWindowId;				//	The sub window
	int				mXDistance;				//	Border size from the left of each column
	int				mYDistance;				//	Use this to avoid clashing with other text at top of screen

private:	//	Functions
	
	int				GetColumnWidth(int columns);
	void				CreateColumns(int columns);
	void				DeleteAll();																							//	Delete all columns

private:	//	General purpose properties

	template <typename T> bool	SetValue(T &var, T value);															//	Allows shorthand in the code
	
public:	// Constructors and initialisers

	//CDashboardV3()																											//	Not using a default constructor
	CDashboardV3(string name, string subWindowName, int columns, int xDistance, int yDistance);
	CDashboardV3(string name, int subWindowId, int columns, int xDistance, int yDistance);
	~CDashboardV3()										{	DeleteAll();											}

	void				Init(string name, string subWindowName, int columns, int xDistance, int yDistance);
	void				Init(string name, int subWindowId, int columns, int xDistance, int yDistance);
	void				ReDraw()								{	SetXDistance(mXDistance);	}

public:		//	Functions

	bool				IsValid()						{	return(mSubWindowId>=0);	}
	bool				IsValid(int index)			{	return(IsValid() && index>=0 && index<ArraySize(mColumns));	}

public:		//	Properties

	CDashboardColumn*		Column(int index);				//	Access one column

	string			GetName()						{	return(mName);								}
	
	int				GetSubWindowId()						{	return(mSubWindowId);							}
						// No set subWindow id, only way to change is start again
						
	int				GetColumnCount()				{	return(ArraySize(mColumns));			}
	
	int				GetXDistance()					{	return(mXDistance);						}
	void				SetXDistance(int value);

	int				GetYDistance()					{	return(mYDistance);						}
	void				SetYDistance(int value);
	   
};

//+------------------------------------------------------------------+
//	Constructors and Initialisers
//+------------------------------------------------------------------+

CDashboardV3::CDashboardV3(string name, string subWindowName, int columns, int xDistance, int yDistance)
								:	mSubWindowId(-1)	{
	
	Init(name, subWindowName, columns, xDistance, yDistance);

}

CDashboardV3::CDashboardV3(string name, int subWindowId, int columns, int xDistance, int yDistance)
								:	mSubWindowId(-1)	{
	
	Init(name, subWindowId, columns, xDistance, yDistance);

}

void		CDashboardV3::Init(string name, string subWindowName, int columns, int xDistance, int yDistance) {

	int	subWindowId	=	ChartWindowFind(0, subWindowName);
	Init(name, subWindowId, columns, xDistance, yDistance);

}

void		CDashboardV3::Init(string name, int subWindowId, int columns, int xDistance, int yDistance) {

	//
	//	Standard initialisation routine
	//

	mName						=	name;				//	First set the name of the dashboard
	mSubWindowId					=	subWindowId;			//	Also needed before we do any cleanup
	mXDistance				=	xDistance;
	mYDistance				=	yDistance;
	
	DeleteAll();									//	In case there are any old objects remaining on screen

	CreateColumns(columns);						//	Create the array of column objects
	
}

//+------------------------------------------------------------------+
//	General functions
//+------------------------------------------------------------------+

void		CDashboardV3::DeleteAll() {

	//
	//	Delete all columns
	//
	for (int i = 0; i<ArraySize(mColumns); i++) {
		delete	mColumns[i];								// The destructor in each column takes care of the rest
	}
	ArrayResize(mColumns, 0);

}

//+------------------------------------------------------------------+
//	Create functions
//+------------------------------------------------------------------+

void	CDashboardV3::CreateColumns(int columns) {

	if (!IsValid())	return;
	
	int	columnWidth	=	GetColumnWidth(columns);

	//	Then just loop through and create the columns
	ArrayResize(mColumns, columns);
	for (int i = 0; i<columns; i++) {
		mColumns[i]	=	new CDashboardColumn(mName, mSubWindowId, i, mXDistance+(i*columnWidth), mYDistance);
	}

	return;
	
}    

int	CDashboardV3::GetColumnWidth(int columns) {

	//	First the width of each column
	long screenWidth;
	ChartGetInteger(0, CHART_WIDTH_IN_PIXELS, mSubWindowId, screenWidth);
	int	columnWidth	=	(int)(screenWidth/columns);
	
	return(columnWidth);

}

//+------------------------------------------------------------------+
//	Properties
//+------------------------------------------------------------------+

CDashboardColumn*	CDashboardV3::Column(int index) {

	if (!IsValid(index))	return(NULL);
	
	return(mColumns[index]);
	
}

void		CDashboardV3::SetXDistance(int value) {

	//
	//	Sets the x distance for each column
	//

	if (!IsValid())	return;

	//	First the width of each column
	int	columns	=	ArraySize(mColumns);
	int	columnWidth	=	GetColumnWidth(columns);

	for (int i = 0; i<columns; i++) {
		mColumns[i].SetXDistance(value+(i*columnWidth));
	}
	
	mXDistance	=	value;
	
}

void		CDashboardV3::SetYDistance(int value) {

	//
	//	Sets the y distance for ech column
	//
	if (!IsValid())	return;

	if (mYDistance==value)	return;
	
	for (int i = 0; i<ArraySize(mColumns); i++) {
		mColumns[i].SetYDistance(value);
	}
	
	mYDistance	=	value;
	
}




